%% path variables
addpath ./../ODE_Solvers

%% Lorenz equation
clear all; close all;
f = @(t,x) [-8/3*x(1) + x(2)*x(3);
            -10*x(2) + 10*x(3);
            -x(1)*x(2) + 28*x(2) - x(3)];
        
t_final = 100; x0 = [0; 0; 1e-10];

[t, x] = ode45(f, [0, t_final], x0);
[t1, x1] = rk_4(@lorenz, [0, t_final, 0.05], x0);
%[t1, x1] = rk_4(f, [0, t_final, 0.05], x0);

figure(1); plot(t, x);
figure(2); plot3(x(:,1), x(:,2), x(:,3)); grid
axis([10 42 -20 20 -20 25]); 

figure(3); plot(t1, x1);
figure(4); plot3(x1(:,1), x1(:,2), x1(:,3)); grid
axis([10 42 -20 20 -20 25]); 

%% Lorenz with parameters
clear all; close all;
f = @(t, x, beta, rho, sigma) [-beta*x(1) + x(2)*x(3);
                                -rho*x(2) + rho*x(3);
                                -x(1)*x(2) + sigma*x(2) - x(3)];
t_final = 100; x0 = [0; 0; 1e-10];
%b1 = 8/3; rc1 = 10; s1 = 28;
b1 = 2; r1 = 5; s1 = 20;
[t, x] = ode45(f, [0, t_final], x0, [], b1, r1, s1);
figure(1); plot(t, x);
figure(2); plot3(x(:,1), x(:,2), x(:,3)); grid

% to use our rk_4, for instance
g = @(t, x) f(t, x, b1, r1, s1);
[t1, x1] = rk_4(g, [0, t_final, 0.01], x0);
figure(3); plot(t1, x1);
figure(4); plot3(x1(:,1), x1(:,2), x1(:,3)); grid



%% Van der Pol Equation
clear all; close all;
f = @(t, x, mu) [x(2); -mu*(x(1)^2 - 1)*x(2) - x(1)];
x0 = [-0.2; -0.7]; t_final = 20;
mu = 1; [t1, y1] = ode45(f, [0, t_final], x0, [], mu);
mu = 2; [t2, y2] = ode45(f, [0, t_final], x0, [], mu);
plot(t1,y1, t2,y2,':')
figure; plot(y1(:,1), y1(:,2), y2(:,1), y2(:,2), ':')



%% Satellite (Apollo)
clear all; close all;
x0 = [1.2; 0; 0; -1.04935751];
tic, [t,y] = ode45(@satellite, [0, 20], x0); toc
length(t), figure(1); plot(y(:,1), y(:,3))
%%
options = odeset; options.RelTol = 1e-6;
tic, [t1,y1] = ode45(@satellite, [0, 20], x0, options); toc
length(t1), figure(2); plot(y1(:,1), y1(:,3))

figure(3); plot(t1(1:end-1), diff(t1)), min(diff(t1))
%%
tic, [t2,y2] = rk_4(@satellite, [0, 20, 0.01], x0); toc
length(t2), figure(4); plot(y2(:,1), y2(:,3))

tic, [t3,y3] = rk_4(@satellite, [0, 20, 0.001], x0); toc
length(t3), figure(5); plot(y3(:,1), y3(:,3))


%% Stiff Problem
% Van der Pol (revisited)
clear all; close all;
f = @(t, x, mu) [x(2); -mu*(x(1)^2 - 1)*x(2) - x(1)];
x0 = [2; 0]; t_final = 3000;
h_opt = odeset; h_opt.RelTol = 1e-6;

tic,
mu = 1000; [t, y] = ode15s(f, [0, t_final], x0, h_opt, mu); toc
figure(1); plot(t,y(:,1))
figure(2); plot(t,y(:,2))
figure(3); plot(y(:,1), y(:,2))

%% Another Stiff Problem
clear all; close all;
f = @(t, x) [-21, 19, -20;
             19, -21, 20;
             40, -40, -40]*x;

opt = odeset; opt.RelTol = 1e-6;
tic, [t, y] = ode45(f, [0,1], [1;0;-1], opt); toc
x1 = exp(-2*t); 
x2 = exp(-40*t).*cos(40*t);
x3 = exp(-40*t).*sin(40*t);
y1 = [0.5*x1 + 0.5*x2 + 0.5*x3, ...
      0.5*x1 - 0.5*x2 - 0.5*x3, ...
      -x2 + x3];
figure(1); plot(t, y, t, y1, ':')

tic, [t2, y2] = rk_4(f, [0,1,0.01], [1;0;-1]); toc
figure(2); plot(t, y1, t2, y2, ':')


%% (Easy) Implicit Equation
clear all; close all;

f = @(t,x) inv( [sin(x(1)), cos(x(2));
                 -cos(x(2)), sin(x(1))] ) ...
           * [1 - x(1); -x(2)]; 
opt = odeset; opt.RelTol = 1e-6;
[t,x] = ode45(f, [0,10], [0; 0], opt);
plot(t,x)

%% (Difficult) Implicit Equation
clear all; close all;
% complicated function: complicated_f
[t,x] = ode15s(@complicated_f, [0,2], [1,0,0,1]);
%[t1,x1] = euler_e(@complicated_f, [0,2,0.1], [1,0,0,1]);
%[t1,x1] = rk_4(@complicated_f, [0,2,0.1], [1,0,0,1]);
%[t1,x1] = rk_3o(@complicated_f, [0,2,0.1], [1,0,0,1]);
%[t1,x1] = ab3_rk3(@complicated_f, [0,2,0.1], [1,0,0,1]);
[t1,x1] = abm3pc(@complicated_f, [0,2,0.5], [1,0,0,1]);
figure(1); plot(t, x, t1, x1, ':')
%figure(2); plot(t1, x1)


%% Mass-Spring System
clear all; close all;
m = [10, 2]; s = [100, 120];
f = @(t,u) [ u(2); -s(1)/m(1)*u(1) + s(2)/m(1)*(u(3) - u(1));
             u(4); -s(2)/m(2)*(u(3) - u(1))];
%[t, x] = euler_e(f, [0, 5, 0.05], [0.5, 0, 0.25, 0]);
[t, x] = rk_4(f, [0, 5, 0.05], [0.5, 0, 0.25, 0]);
%[t, x] = ode45(f, [0, 25], [0.5, 0, 0.25, 0]);
%[t, x] = ode23s(f, [0, 5], [.5, 0, .25, 0]);
%[t, x] = abm3pc(f, [0, 5, 0.05], [.5, 0, .25, 0]);

figure(1); plot(t,x)
figure(2); plot(t, x(:,1), t, x(:,3))




%% Calculate and Check
check_me = lorenz(3, [2,3,4]')

%% Some Functions for ODEs
function dx = lorenz(t, x)
    dx = [-8/3*x(1) + x(2)*x(3);
            -10*x(2) + 10*x(3);
            -x(1)*x(2) + 28*x(2) - x(3)];
end


function dx = satellite(t, x)
    mu = 1/82.45; mu1 = 1 - mu;
    r1 = sqrt((x(1) + mu)^2 + x(3)^2);
    r2 = sqrt((x(1) - mu1)^2 + x(3)^2);
    
    dx = [x(2);
          2*x(4) + x(1) - mu1*(x(1) + mu)/r1^3 ...
            - mu*(x(1) - mu1)/r2^3;
          x(4);
          -2*x(2) + x(3) - mu1*x(3)/r1^3 ...
            - mu*x(3)/r2^3];
end

function dy = complicated_f(t, x)
    dx = @(p, x) [p(1)*sin(x(4)) + p(2)^2 ...
                      + 2*x(1)*x(3)*exp(-x(2)) - x(1)*p(1)*x(4);
                  x(1)*p(1)*p(2) + cos(p(2)) ... 
                      - 3*x(3)*x(2)*exp(-x(1))];
    f_optim = optimset; f_optim.Display = 'off';
    dx1 = fsolve(dx, x([1,3]), f_optim, x);
    dy = [x(2); dx1(1); x(4); dx1(2)];
end
